-----------------------------
-- Get the addon table
-----------------------------

local AddonName = select(1, ...);
local iQuest = LibStub("AceAddon-3.0"):GetAddon(AddonName);

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

---------------------------------
-- The configuration table
---------------------------------

local function CreateConfig()
	CreateConfig = nil; -- we just need this function once, thus removing it from memory.

	local db = {
		type = "group",
		name = AddonName,
		order = 1,
		childGroups = "tab",
		args = {
			GroupGeneral = {
				type = "group",
				name = L["General"],
				order = 1,
				args = {
					EmptyLine1 = {
						type = "header",
						name = L["Message Output"],
						order = 1,
					},
					OutputMessages = {
						type = "toggle",
						name = L["Display quest progression messages"],
						order = 2,
						width = "full",
						get = function()
							return iQuest.db.OutputMessages;
						end,
						set = function(info, value)
							iQuest.db.OutputMessages = value;
						end,
					},
					OutputSoundMessages = {
						type = "toggle",
						name = L["Play sound when progressing or finishing a quest"],
						order = 3,
						width = "full",
						get = function()
							return iQuest.db.OutputSoundMessages;
						end,
						set = function(info, value)
							iQuest.db.OutputSoundMessages = value;
						end,
					},
					EmptyLine3 = {
						type = "description",
						name = " ",
						order = 4,
					},
					EmptyLine2 = {
						type = "description",
						name = L["IMPORTANT: If you enable this option, iQuest will change default UI elements (here: Errors Frame). If you are addons running which work on it as well, you may encounter errors."],
						order = 5,
					},
					HookUIErrorsFrame = {
						type = "toggle",
						name = L["Filter Blizzard quest progression messages"],
						order = 6,
						width = "full",
						get = function()
							return iQuest.db.HookUIErrorsFrame;
						end,
						set = function(info, value)
							iQuest.db.HookUIErrorsFrame = value;
							iQuest:UpdateHooks();
						end,
					},
					FilterErrorMessageSpam = {
						type = "toggle",
						name = L["Filter error message spam"],
						order = 7,
						width = "full",
						get = function()
							return iQuest.db.FilterErrorMessageSpam;
						end,
						set = function(info, value)
							iQuest.db.FilterErrorMessageSpam = value;
						end,
					},
					EmptyLine4 = {
						type = "header",
						name = L["Blizzard Quest UI Elements"],
						order = 8,
					},
					AddTagsToQuestlog = {
						type = "toggle",
						name = L["Add tags to Blizzard questlog"],
						order = 9,
						width = "full",
						get = function()
							return iQuest.db.AddTagsToQuestlog;
						end,
						set = function(info, value)
							iQuest.db.AddTagsToQuestlog = value;
						end,
					},
					AddTagsToQuestGivers = {
						type = "toggle",
						name = L["Add tags to quest givers"],
						order = 9,
						width = "full",
						get = function()
							return iQuest.db.AddTagsToQuestGivers;
						end,
						set = function(info, value)
							iQuest.db.AddTagsToQuestGivers = value;
						end,
					},
				},
			},
			GroupFeed = {
				type = "group",
				name = L["Feed"],
				order = 2,
				args = {
					ShowNumMaxQuestLog = {
						type = "toggle",
						name = L["Show max quests"],
						order = 1,
						get = function()
							return iQuest.db.ShowNumMaxQuestLog;
						end,
						set = function(info, value)
							iQuest.db.ShowNumMaxQuestLog = value;
							iQuest:UpdateFeed();
						end,
					},
					EncolorQuestsBy = {
						type = "select",
						name = _G.COLOR,
						order = 2,
						get = function()
							return iQuest.db.EncolorQuestsBy;
						end,
						set = function(info, value)
							iQuest.db.EncolorQuestsBy = value;
							iQuest:UpdateFeed();
						end,
						values = {
							[1] = _G.NONE,
							[2] = L["Threshold"]
						},
					},
					EmptyLine1 = {
						type = "header",
						name = "",
						order = 3,
					},
					DailyDisplayType = {
						type = "select",
						name = L["Daily quest display type"],
						order = 4,
						get = function()
							return iQuest.db.DailyDisplayType;
						end,
						set = function(info, value)
							iQuest.db.DailyDisplayType = value;
							iQuest:UpdateFeed();
						end,
						values = {
							[1] = _G.NONE,
							[2] = L["Left Dailys"],
							[3] = L["Solved Dailys"]
						},
						style = "radio",
					},
					ShowNumMaxDailys = {
						type = "toggle",
						name = L["Show max dailys"],
						order = 5,
						get = function()
							return iQuest.db.ShowNumMaxDailys;
						end,
						set = function(info, value)
							iQuest.db.ShowNumMaxDailys = value;
							iQuest:UpdateFeed();
						end,
					},
					EncolorDailysBy = {
						type = "select",
						name = _G.COLOR,
						order = 6,
						get = function()
							return iQuest.db.EncolorDailysBy;
						end,
						set = function(info, value)
							iQuest.db.EncolorDailysBy = value;
							iQuest:UpdateFeed();
						end,
						values = {
							[1] = _G.NONE,
							[2] = L["Daily Blue"]
						},
					},
					DisableDailysNoMaxLevel = {
						type = "toggle",
						name = L["Hide dailys when not at max level"],
						order = 7,
						width = "full",
						get = function()
							return iQuest.db.DisableDailysNoMaxLevel;
						end,
						set = function(info, value)
							iQuest.db.DisableDailysNoMaxLevel = value;
							iQuest:UpdateFeed();
						end,
					},
				},
			},
			--GroupTooltip = {
			--	type = "group",
			--	name = "Tooltip",
			--	order = 3,
			--	args = {
			--		
			--	},
			--},
			GroupQuestTracker = {
				type = "group",
				name = L["Quest Tracker"],
				order = 10,
				args = {
					EmptyLine1 = {
						type = "description",
						name = L["IMPORTANT: If you enable this option, iQuest will change default UI elements (here: Blizzard Quest Tracker). If you are addons running which work on it as well, you may encounter errors."],
						order = 1,
					},
					ColorizeQuestTracker = {
						type = "toggle",
						name = L["Change Quest Tracker"],
						order = 2,
						width = "full",
						get = function()
							return iQuest.db.ColorizeQuestTracker;
						end,
						set = function(info, value)
							iQuest.db.ColorizeQuestTracker = value;
							iQuest:UpdateHooks();
						end,
					},
					EmptyLine2 = {
						type = "description",
						name = " ",
						order = 3,
					},
					EmptyLine2 = {
						type = "header",
						name = L["Quest Options"],
						order = 4,
					},
					Tracker_EncolorQuestHeader = {
						type = "toggle",
						name = L["Colorize Names"],
						order = 5,
						width = "full",
						get = function()
							return iQuest.db.Tracker_EncolorQuestHeader;
						end,
						set = function(info, value)
							iQuest.db.Tracker_EncolorQuestHeader = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_EncolorQuestObjectives = {
						type = "toggle",
						name = L["Colorize Objectives"],
						order = 6,
						width = "full",
						get = function()
							return iQuest.db.Tracker_EncolorQuestObjectives;
						end,
						set = function(info, value)
							iQuest.db.Tracker_EncolorQuestObjectives = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_AddQuestTag = {
						type = "toggle",
						name = L["Add tags to quests"],
						desc = L["Quest tags display if the quest is elite, a Dungeon quest, etc."],
						order = 7,
						width = "full",
						get = function()
							return iQuest.db.Tracker_AddQuestTag;
						end,
						set = function(info, value)
							iQuest.db.Tracker_AddQuestTag = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_RemoveOnDone = {
						type = "toggle",
						name = L["Remove quest from tracker when completed"],
						order = 8,
						width = "full",
						get = function()
							return iQuest.db.Tracker_RemoveOnDone;
						end,
						set = function(info, value)
							iQuest.db.Tracker_RemoveOnDone = value;
						end,
					},
					EmptyLine3 = {
						type = "header",
						name = L["Achievement Options"],
						order = 9,
					},
					Tracker_EncolorAcmHeader = {
						type = "toggle",
						name = L["Colorize Names"],
						order = 10,
						width = "full",
						get = function()
							return iQuest.db.Tracker_EncolorAcmHeader;
						end,
						set = function(info, value)
							iQuest.db.Tracker_EncolorAcmHeader = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_EncolorAcmObjectives = {
						type = "toggle",
						name = L["Colorize Objectives"],
						order = 11,
						width = "full",
						get = function()
							return iQuest.db.Tracker_EncolorAcmObjectives;
						end,
						set = function(info, value)
							iQuest.db.Tracker_EncolorAcmObjectives = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_AddZoneAcmToTracker = {
						type = "toggle",
						name = L["Add zone quest achievement to tracker"],
						order = 12,
						width = "full",
						get = function()
							return iQuest.db.Tracker_AddZoneAcmToTracker;
						end,
						set = function(info, value)
							iQuest.db.Tracker_AddZoneAcmToTracker = value;
							iQuest:ZoneAchievementChange((value == true and 1 or nil), 1);
						end,
					},
					EmptyLine4 = {
						type = "header",
						name = L["Font Options"],
						order = 12,
					},
					Tracker_UseFontFamily = {
						type = "toggle",
						name = L["Replace Font"],
						order = 13,
						get = function()
							return iQuest.db.Tracker_UseFontFamily;
						end,
						set = function(info, value)
							iQuest.db.Tracker_UseFontFamily = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_FontFamily = {
						type = "select",
						name = L["Font"],
						order = 14,
						values = LibStub("LibSharedMedia-3.0"):HashTable("font"),
						dialogControl = "LSM30_Font",
						get = function()
							return iQuest.db.Tracker_FontFamily;
						end,
						set = function(info, value)
							iQuest.db.Tracker_FontFamily = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_FontOutline = {
						type = "select",
						name = L["Outline"],
						order = 15,
						values = {
							[""] = _G.NONE,
							["OUTLINE"] = L["Outline"],
							["THICKOUTLINE"] = L["Thick Outline"]
						},
						get = function()
							return iQuest.db.Tracker_FontOutline;
						end,
						set = function(info, value)
							iQuest.db.Tracker_FontOutline = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_UseFontSize = {
						type = "toggle",
						name = L["Adjust Font Size"],
						order = 16,
						get = function()
							return iQuest.db.Tracker_UseFontSize;
						end,
						set = function(info, value)
							iQuest.db.Tracker_UseFontSize = value;
							_G.WatchFrame_Update();
						end,
					},
					Tracker_FontSize = {
						type = "range",
						name = L["Font Size"],
						order = 17,
						min = 8,
						max = 16,
						step = 1,
						get = function()
							return iQuest.db.Tracker_FontSize;
						end,
						set = function(info, value)
							iQuest.db.Tracker_FontSize = value;
							_G.WatchFrame_Update();
						end,
					},
				},
			},
		}
	};
	
	return db;
end

function iQuest:CreateDB()
	iQuest.CreateDB = nil;
	
	return { profile = {
		ShowNumMaxQuestLog = false,
		EncolorQuestsBy = 2,
		ShowNumMaxDailys = false,
		DailyDisplayType = 3,
		EncolorDailysBy = 1,
		DisableDailysNoMaxLevel = true,
		ColorizeQuestTracker = true,
		Tracker_EncolorQuestHeader = true,
		Tracker_EncolorQuestObjectives = true,
		Tracker_AddQuestTag = true,
		Tracker_EncolorAcmHeader = true,
		Tracker_EncolorAcmObjectives = true,
		Tracker_AddZoneAcmToTracker = false,
		Tracker_UseFontFamily = false,
		Tracker_FontFamily = "Friz Quadrata TT",
		Tracker_UseFontSize = false,
		Tracker_FontSize = 12,
		Tracker_FontOutline = "",
		Tracker_RemoveOnDone = false,
		OutputMessages = true,
		OutputSoundMessages = true,
		HookUIErrorsFrame = true,
		FilterErrorMessageSpam = false,
		AddTagsToQuestlog = true,
		AddTagsToQuestGivers = true,
	}};
end

function iQuest:OpenOptions()
	_G.InterfaceOptionsFrame_OpenToCategory(AddonName);
end

LibStub("AceConfig-3.0"):RegisterOptionsTable(AddonName, CreateConfig);
LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AddonName);
_G.SlashCmdList["IQUEST"] = iQuest.OpenOptions;
_G["SLASH_IQUEST1"] = "/iquest";