-----------------------------
-- Get the addon table
-----------------------------

local AddonName = select(1, ...);
local iQuest = LibStub("AceAddon-3.0"):GetAddon(AddonName);

--local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);
local BL = LibStub("LibBabble-Zone-3.0"):GetLookupTable();

local za, acm;

function iQuest:CheckZoneAchievement(event)
	if( event == "PLAYER_ENTERING_WORLD" ) then
		self:InitZoneAchievementTable();
		self:UnregisterEvent("PLAYER_ENTERING_WORLD");
	end
	
	local zone = _G.GetRealZoneText();
	self:ZoneAchievementChange();
	
	if( not zone or not za[zone] ) then
		acm = nil;
		return;
	end
	
	acm = za[zone];
	self:ZoneAchievementChange(1);
end

function iQuest:ZoneAchievementChange(val, force)
	if( not acm or (not force and self.db.Tracker_AddZoneAcmToTracker == false) ) then
		return;
	end
	
	if( val ) then
		_G.AddTrackedAchievement(acm);
	else
		_G.RemoveTrackedAchievement(acm);
	end
end

local function add_za(place, acmID)
	local _, _, _, complete = _G.GetAchievementInfo(acmID);
	if( complete ~= true ) then
		za[BL[place]] = acmID;
	end
end

function iQuest:InitZoneAchievementTable()
self.InitZoneAchievementTable = nil;

local isAlly, isHorde;
local faction = _G.UnitFactionGroup("player");

if( faction == "Alliance" ) then
	isAlly = 1;
end

if( faction == "Horde" ) then
	isHorde = 1;
end

za = {};

-- Eastern Kingdoms
add_za("The Cape of Stranglethorn", 4905);
add_za("Arathi Highlands", 4896);
add_za("The Hinterlands", 4897);
add_za("Northern Stranglethorn", 4906);
add_za("Badlands", 4900);
add_za("Swamp of Sorrows", 4904);
add_za("Blasted Lands", 4909);
add_za("Western Plaguelands", 4893);
add_za("Eastern Plaguelands", 4892);
add_za("Burning Steppes", 4901);
add_za("Searing Gorge", 4910);

if( isAlly ) then
	add_za("Duskwood", 4907);
	add_za("Redridge Mountains", 4902);
	add_za("Wetlands", 4898);
	add_za("Loch Modan", 4899);
	add_za("Westfall", 4903);
end

if( isHorde ) then
	add_za("Silverpine Forest", 4894);
	add_za("Hillsbrad Foothills", 4895);
	add_za("Ghostlands", 4908);
end

-- Kalimdor
add_za("Un'Goro Crater", 4939);
add_za("Felwood", 4931);
add_za("Desolace", 4930);
add_za("Silithus", 4934);
add_za("Tanaris", 4935);
add_za("Thousand Needles", 4938);
add_za("Winterspring", 4940);

if( isAlly ) then
	add_za("Bloodmyst Isle", 4926);
	add_za("Ashenvale", 4925);
	add_za("Stonetalon Mountains", 4936);
	add_za("Southern Barrens", 4937);
	add_za("Darkshore", 4928);
	add_za("Feralas", 4932);
	add_za("Dustwallow Marsh", 4929);
end

if( isHorde ) then
	add_za("Northern Barrens", 4933);
	add_za("Stonetalon Mountains", 4980);
	add_za("Southern Barrens", 4981);
	add_za("Azshara", 4927);
	add_za("Ashenvale", 4976);
	add_za("Feralas", 4979);
	add_za("Dustwallow Marsh", 4978);
end

-- Draenor
add_za("Netherstorm", 1194);
add_za("Zangarmarsh", 1190);
add_za("Shadowmoon Valley", 1195);
add_za("Blade's Edge Mountains", 1276);

if( isAlly ) then
	add_za("Hellfire Peninsula", 1189);
	add_za("Nagrand", 1192);
	add_za("Terokkar Forest", 1191);
end

if( isHorde ) then
	add_za("Hellfire Peninsula", 1271);
	add_za("Nagrand", 1273);
	add_za("Terokkar Forest", 1272);
end

-- Northrend
add_za("Sholazar Basin", 39);
add_za("Zul'Drak", 36);
add_za("The Storm Peaks", 38);
add_za("Icecrown", 40);

if( isAlly ) then
	add_za("Howling Fjord", 34);
	add_za("Grizzly Hills", 37);
	add_za("Dragonblight", 35);
	add_za("Borean Tundra", 33);
end

if( isHorde ) then
	add_za("Howling Fjord", 1356);
	add_za("Grizzly Hills", 1357);
	add_za("Dragonblight", 1359);
	add_za("Borean Tundra", 1358);
end

-- Cataclysm
add_za("Mount Hyjal", 4870);
add_za("Uldum", 4872);
add_za("Deepholm", 4871);

if( isAlly ) then
	add_za("Vashj'ir", 4869);
	add_za("Twilight Highlands", 4873);
end

if( isHorde ) then
	add_za("Vashj'ir", 4982);
	add_za("Twilight Highlands", 5501);
end

-- Pandaria
--add_za("Jade Forest", 0x0);
--add_za("Valley of the Four Winds", 0x0);
--add_za("Kun-Lai Summit", 0x0);
--add_za("Townlong Steppes", 0x0);
--add_za("Krasarang Wilds", 0x0);
--add_za("Dread Wastes", 0x0);
--add_za("Vale of Eternal Blossoms", 0x0);

BL = nil;
end