-----------------------------------
-- Setting up scope and libs
-----------------------------------

local AddonName = select(1, ...);
iQuest = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceHook-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

local LibQTip = LibStub("LibQTip-1.0");
local LibTourist = LibStub("LibTourist-3.0");
local LibCrayon = LibStub("LibCrayon-3.0");

local _G = _G;

-----------------------------------------
-- Variables, functions and colors
-----------------------------------------

local Tooltip; -- our tooltip

local COLOR_GOLD = "|cfffed100%s|r";

-- I borrowed the following four lines from LibQuixote as they do their job awesome ;)
local pattern_object = "^".._G.QUEST_OBJECTS_FOUND:gsub('(%%%d?$?.)',  '(.+)').."$"; --QUEST_OBJECTS_FOUND = "%s: %d/%d"
local pattern_monster= "^".._G.QUEST_MONSTERS_KILLED:gsub('(%%%d?$?.)','(.+)').."$"; --QUEST_MONSTERS_KILLED = "%s slain: %d/%d"
local pattern_faction= "^".._G.QUEST_FACTION_NEEDED:gsub('(%%%d?$?.)', '(.+)').."$"; --QUEST_FACTION_NEEDED = "%s: %s / %s"
local pattern_player = "^".._G.QUEST_PLAYERS_KILLED:gsub('(%%%d?$?.)', '(.+)').."$"; --QUEST_PLAYERS_KILLED = "Players slain: %d/%d"

-----------------------------
-- Setting up the feed
-----------------------------

iQuest.Feed = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",
	icon = "Interface\\MINIMAP\\TRACKING\\Class",
});

iQuest.Feed.OnClick = function(_, button)
	if( button == "LeftButton" ) then
		_G.ToggleFrame(_G.QuestLogFrame);
	elseif( button == "RightButton" ) then
		iQuest:OpenOptions();
	end
end

iQuest.Feed.OnEnter = function(anchor)
	--for k, v in LibQTip:IterateTooltips() do
	--	if( type(k) == "string" and strsub(k, 1, 6) == "iSuite" ) then
	--		v:Release(k);
	--	end
	--end
		
	--Tooltip = LibQTip:Acquire("iSuite"..AddonName);
	--Tooltip:SmartAnchorTo(anchor);
	--iQuest:UpdateTooltip();
	--Tooltip:Show();
end

iQuest.Feed.OnLeave = function()
	--Tooltip:Release();
end

----------------------
-- OnInitialize
----------------------

function iQuest:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("iQuestDB", self:CreateDB(), "Default").profile;
	
	-- for our quest scanning funcs
	self:RegisterEvent("QUEST_LOG_UPDATE", "UpdateQuestlog");
	
	-- for scanning zone achievements
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "CheckZoneAchievement");
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "CheckZoneAchievement");
	
	self:UpdateHooks();
end

---------------------
-- UpdateHooks
---------------------

function iQuest:UpdateHooks()
	-- Hook Errors Frame
	if( self.db.HookUIErrorsFrame == true ) then
		if( not self:IsHooked(_G.UIErrorsFrame, "AddMessage") ) then
			self:RawHook(_G.UIErrorsFrame, "AddMessage", "Hook_ErrorAddMessage", true);
		end
	else
		if( self:IsHooked(_G.UIErrorsFrame, "AddMessage") ) then
			self:Unhook(_G.UIErrorsFrame, "AddMessage");
		end
	end
	
	-- Hook Watch Frame
	if( not _G.IsAddOnLoaded("!ColorizedQuestTracker") ) then -- prevents double-hooking if the mod is loaded
		if( self.db.ColorizeQuestTracker == true ) then
			if( not self:IsHooked("WatchFrame_Update") ) then
				self:SecureHook("WatchFrame_Update", "Hook_WatchFrame");
				self:SecureHook("WatchFrameLinkButtonTemplate_Highlight", "Hook_WatchFrame");
				self:SecureHook("WatchFrame_UpdateTimedAchievements", "Hook_WatchFrame");
			end
		else
			if( self:IsHooked("WatchFrame_Update") ) then
				self:Unhook("WatchFrame_Update");
				self:Unhook("WatchFrameLinkButtonTemplate_Highlight");
				self:Unhook("WatchFrame_UpdateTimedAchievements");
			end
		end
		_G.WatchFrame_Update();
	end
	
	-- Hook Questlog Frame
	if( self.db.AddTagsToQuestlog == true ) then
		if( not self:IsHooked("QuestLog_Update") ) then
			self:SecureHook("QuestLog_Update", "Hook_QuestLogUpdate");
		end
	else
		if( self:IsHooked("QuestLog_Update") ) then
			self:Unhook("QuestLog_Update");
		end
	end
	
	-- Hook Quest Giver Frame
	if( self.db.AddTagsToQuestGivers == true ) then
		self:RegisterEvent("GOSSIP_SHOW", "Hook_QuestGossip");
		self:RegisterEvent("QUEST_GREETING", "Hook_QuestGossip");
	else
		self:UnregisterEvent("GOSSIP_SHOW");
		self:UnregisterEvent("QUEST_GREETING");
	end
end

do
	local lastErrorMsg;
	function iQuest:Hook_ErrorAddMessage(...)
		local msg = select(2, ...) or "";
		
		if( self.db.FilterErrorMessageSpam == true and lastErrorMsg == msg ) then
			return;
		end
		lastErrorMsg = msg;
		
		if( msg:find(pattern_object) and not msg:find("^%(") ) then
			return;
		elseif( msg:find(pattern_monster) ) then
			return;
		end
		
		self.hooks[_G.UIErrorsFrame].AddMessage(...);
	end
	
	local qtitle, qlevel, qtag, qgroup, isHeader, isCollapsed, isComplete, isDaily, qid, qevent, q_;
	function iQuest:Hook_QuestLogUpdate()
		for i = 1, #_G.QuestLogScrollFrame.buttons do
			local line = _G["QuestLogScrollFrameButton"..i];
			
			if( line and line:GetText() ) then
				qtitle, qlevel, qtag, qgroup, isHeader, isCollapsed, isComplete, isDaily, qid, qevent, q_ = _G.GetQuestLogTitle(i);
				if( not isHeader and qtitle ) then
					if( not isComplete ) then
						_G["QuestLogScrollFrameButton"..i.."Tag"]:SetText("");
					end
					
					line:SetText(self:GenerateQuestTag(qtag, isDaily, qlevel)..qtitle);
					_G.QuestLogTitleButton_Resize(line);
				end
			end
		end
	end
	
	local function ParseGossip(step, numButton, ...)
		local toValue = select("#", ...);
		
		for i = 1, toValue, step do
			local btn = _G["GossipTitleButton"..numButton];
			btn:SetText(iQuest:GenerateQuestTag("", select(i +3, ...), select(i +1, ...))..btn:GetText());
			numButton = numButton + 1;
		end
		
		return numButton + 1;
	end
	
	function iQuest:Hook_QuestGossip(event)
		if( event == "GOSSIP_SHOW" ) then
			local numButton = 1;
			
			if( _G.GetGossipAvailableQuests() ) then
				numButton = ParseGossip(5, numButton, _G.GetGossipAvailableQuests() );
			end
			
			if _G.GetGossipActiveQuests() then
				numButton = ParseGossip(4, numButton, _G.GetGossipActiveQuests() );
			end
		elseif( event == "QUEST_GREETING" ) then
			local numActive, numAvail = _G.GetNumActiveQuests(), _G.GetNumAvailableQuests();
			local title, level, btn;
			local o, GetTitle, GetLevel = 0, _G.GetActiveTitle, _G.GetActiveLevel;
			
			for i = 1, numActive + numAvail do
				if( i == numActive + 1 ) then
					o, GetTitle, GetLevel = numActive, _G.GetAvailableTitle, _G.GetAvailableLevel;
				end
				
				title, level = GetTitle(i -o), GetLevel(i -o);
				btn = _G["QuestTitleButton"..i];
				btn:SetText(self:GenerateQuestTag("", nil, level)..title);
			end
		end
	end
end

------------------------
-- UpdateQuestlog
------------------------

iQuest.Active = 0;
iQuest.Completed = 0;

do	
	local isRunning; -- prevents from updating too often
	local Entries, Quests, Completed;
	local qtitle, qlevel, qtag, qgroup, isHeader, isCollapsed, isComplete, isDaily, qid, qevent, q_;
	
	function iQuest:UpdateQuestlog()
		if( isRunning ) then
			return;
		end
		isRunning = 1;
		
		local newImage = {};
		Completed = 0;
		Entries, Quests = _G.GetNumQuestLogEntries();
		
		for i = 1, Entries, 1 do -- loop thru all entries in questlog
			qtitle, qlevel, qtag, qgroup, isHeader, isCollapsed, isComplete, isDaily, qid, qevent, q_ = _G.GetQuestLogTitle(i);
			
			if( not isHeader ) then -- headers give zone or event names
				if( isComplete ) then
					Completed = Completed + 1;
				end
				
				local o = {};
				local numObjectives = _G.GetNumQuestLeaderBoards(i);
				
				if( numObjectives and numObjectives > 0 ) then
					for y = 1, numObjectives, 1 do
						local info, otype, done = _G.GetQuestLogLeaderBoard(y, i);
						local numCurrent, numNeeded, name;
						
						if( otype == "monster" ) then
							name, numCurrent, numNeeded = info:match(pattern_monster);
							if( name == nil or numCurrent == nil or numNeeded == nil ) then
								-- as stated in LibQuixote, sometimes quests are listed as monster, but need to be checked with object pattern
								name, numCurrent, numNeeded = info:match(pattern_object);
							end
						elseif( otype == "item" or otype == "object" ) then
							name, numCurrent, numNeeded = info:match(pattern_object);
						elseif( otype == "reputation" ) then
							name, numCurrent, numNeeded = info:match(pattern_faction);
						elseif( otype == "event" or otype == "log" or otype == "spell" ) then
							name = otype;
							numNeeded = 1;
							numCurrent = done and 1 or 0;
						elseif( otype == "player" ) then
							numCurrent, numNeeded = info:match(pattern_player);
							name = _G.COMBATLOG_FILTER_STRING_HOSTILE_PLAYERS; -- enemy player
						end
						
						numNeeded = tonumber(numNeeded);
						numCurrent = tonumber(numCurrent);
						
						o[name] = {numCurrent, numNeeded, otype, done};
					end
				end
				
				local q = {o, i, qtitle, isComplete};
				newImage[tostring(qid)] = q;
			end			
		end
		
		isRunning = nil;
		self.Active = Quests;
		self.Completed = Completed;
		
		self:CompareQuestlogImages(newImage);
		self.QuestlogImage = newImage;
		
		self:UpdateFeed();
	end
end

function iQuest:CompareQuestlogImages(img)
	-- possibly the first boot of iQuest, so no questlog image is available!
	if( not self.QuestlogImage ) then
		return;
	end
	
	local imgold = self.QuestlogImage;
	local questold, obj, objold;
	
	for qid, quest in pairs(img) do
		questold = imgold[qid];
		
		-- we need to check if the given quest is in the image to compare
		if( questold ) then
			-- quest is done!
		if ( quest[4] and not questold[4] ) then
			self:QuestProgression(1, quest[2], quest[3]);
		else
			obj = quest[1];
			objold = questold[1];
			
			if( obj and objold ) then
				for k, v in pairs(obj) do
					if( objold[k] ) then
						if( v[4] and not objold[k][4] ) then
							self:QuestProgression(2, quest[2], quest[3], k, unpack(v));
						elseif( v[1] > objold[k][1] ) then
							self:QuestProgression(3, quest[2], quest[3], k, unpack(v));
						end
					end -- end if
				end -- end for
			end -- end if
		end -- end if	
		end -- end if
	end -- end for
end -- end function

function iQuest:QuestProgression(mode, id, title, objective, current, needed, qtype)
	-- mode = 1: Quest done
	-- mode = 2: Quest objective done
	-- mode = 3: Quest objective progression
	
	if( self.db.OutputMessages ) then
		if( mode == 1 or mode == 2 ) then
			_G.UIErrorsFrame:AddMessage((ERR_QUEST_COMPLETE_S):format(mode == 1 and title or "("..title..") "..objective), 0, 1, 0, nil, 3);
		elseif( mode == 3 ) then
			_G.UIErrorsFrame:AddMessage("("..title..") "..objective..": "..current.."/"..needed, 1, 1, 0, nil, 3);
		end
	end
	
	if( self.db.OutputSoundMessages ) then
		if( mode == 1 ) then
			PlaySoundFile("Sound\\Creature\\Peon\\PeonBuildingComplete1.wav"); 
		elseif( mode == 2 ) then
			PlaySoundFile("Sound\\Creature\\Peon\\PeonReady1.wav");
		end
	end
	
	if( self.db.Tracker_RemoveOnDone == true and mode == 1 and _G.IsQuestWatched(id) ) then
		_G.RemoveQuestWatch(id);
		_G.WatchFrame_Update();
	end
end

function iQuest:GenerateQuestTag(questTag, isDaily, level)
	-- this short-tags are spread over the whole world and known by almost everyone.
	-- thanks to your good work, Fubar_QuestsFu team.
	-- I will use them, too! :)
	local newTag;
	
	if( questTag == _G.ELITE ) then
		newTag = "+";
	elseif( questTag == _G.LFG_TYPE_DUNGEON ) then
		newTag = "d";
	elseif( questTag == _G.PVP ) then
		newTag = "p";
	elseif( questTag == _G.RAID ) then
		newTag = "r"
	elseif( questTag == _G.GROUP ) then
		newTag = "g";
	elseif( questTag == _G.PLAYER_DIFFICULTY2 ) then
		newTag = "d+";
	else
		newTag = "";
	end
	if( isDaily ) then
		newTag = newTag .. "\226\128\162";
	end
	
	
	-- event quests level may be -1, so we replace it with a *
	if( level == -1 ) then
		level = "*";
	else
		level = tostring(level);
	end
	
	newTag = ("[%s%s] "):format(level, newTag);
	return newTag;
end

-----------------------------------------
-- Hooking - ColorizeQuestTracker
-----------------------------------------

do
	local function AdjustFont(fontObj)		
		local font, size, flags = fontObj:GetFont();
		
		font = iQuest.db.Tracker_UseFontFamily == true and LibStub("LibSharedMedia-3.0"):Fetch("font", iQuest.db.Tracker_FontFamily)
			or LibStub("LibSharedMedia-3.0"):Fetch("font", LibStub("LibSharedMedia-3.0"):GetDefault("font"));
		
		size = iQuest.db.Tracker_UseFontSize == true and iQuest.db.Tracker_FontSize or 12;
		
		flags = iQuest.db.Tracker_FontOutline;
		
		fontObj:SetFont(font, size, flags);
	end
	
	local function HandleQuest(link)
		local questIndex = _G.GetQuestIndexForWatch(link.index);
		
		if( not questIndex ) then
			return;
		end
		
		local _, level, questTag, suggestedGroup, _, _, isComplete, isDaily, _ = _G.GetQuestLogTitle(questIndex);
		local color = _G.GetQuestDifficultyColor(level);	
		local line;
		
		for index = link.startLine, link.lastLine do
			line = link.lines[index];
			
			if( line ) then
				AdjustFont(line.text);
				
				if( index == link.startLine ) then
					-- header
					if( not string.find(line.text:GetText() or "", "^%[.*%].*") and iQuest.db.Tracker_AddQuestTag ) then
						line.text:SetText( iQuest:GenerateQuestTag(questTag, isDaily, level)..(line.text:GetText() or "") );
					end
					
					if( iQuest.db.Tracker_EncolorQuestHeader ) then
						line.text:SetTextColor(color.r, color.g, color.b);
					end
				else
					if( iQuest.db.Tracker_EncolorQuestObjectives ) then
						local r, g, b;
						
						if( line.text:GetText() == _G.GetQuestLogCompletionText(questIndex) ) then
							r, g, b = LibCrayon:GetThresholdColor(1, 1);
						else
							local _, _, numItems, numNeeded = string.find(line.text:GetText() or "", "([%d]+)/([%d]+).*");
							
							if( numItems and numNeeded ) then
								r, g, b = LibCrayon:GetThresholdColor(tonumber(numItems), tonumber(numNeeded));
							end
						end
						
						line.text:SetTextColor(r or 1, g or 0, b or 0);
					end -- check config
				end -- check start line
			end -- check line
		end -- end for
	end
	
	local function HandleAchievement(link)
		local line;
		
		for index = link.startLine, link.lastLine do
			line = link.lines[index];
			
			if( line ) then
				AdjustFont(line.text);
				
				if( index == link.startLine ) then
					-- header						
					if( iQuest.db.Tracker_EncolorAcmHeader ) then
						line.text:SetTextColor(1, 1, 1);
					end
				else
					if( iQuest.db.Tracker_EncolorAcmObjectives ) then
						local r, g, b;
						
						local _, _, numJobs, numJobsNeeded = string.find(line.text:GetText() or "", "([%d]+)/([%d]+).*");
						local _, _, numGold, numGoldNeeded = string.find(line.text:GetText() or "", "([%d]+)%|T.*%/ ([%d]+)%|TInterface%\\MoneyFrame.*");
						
						-- normal 1/x string
						if( numJobs and numJobsNeeded ) then
							r, g, b = LibCrayon:GetThresholdColor(tonumber(numJobs), tonumber(numJobsNeeded));							
						-- money achievement - really dirty way but I don't want to loop thru the achievement criterias	
						elseif( numGold and numGoldNeeded ) then
							r, g, b = LibCrayon:GetThresholdColor(tonumber(numGold) *10000, tonumber(numGoldNeeded) *10000);
						end
						
						line.text:SetTextColor(r or 1, g or 0, b or 0);
					end -- check config
				end -- check start line
			end -- check line
		end -- end for
	end

	local link;
	function iQuest:Hook_WatchFrame()
		for i = 1, #_G.WATCHFRAME_LINKBUTTONS do
			link = _G.WATCHFRAME_LINKBUTTONS[i];
			if( link.type == "QUEST" ) then
				HandleQuest(link);
			elseif( link.type == "ACHIEVEMENT" ) then
				HandleAchievement(link);
			end
		end
	end
end

--------------------
-- UpdateFeed
--------------------

function iQuest:UpdateFeed()
	self.Feed.text = ("%d/%d"):format(self.Completed, self.Active);
	
	if( self.db.ShowNumMaxQuestLog ) then
		self.Feed.text = self.Feed.text.."/".._G.MAX_QUESTS;
	end
	
	if( self.db.EncolorQuestsBy == 2 ) then
		self.Feed.text = ("|cff%s%s|r"):format(LibCrayon:GetThresholdHexColor(self.Completed, self.Active), self.Feed.text)
	end
	
	if( self.db.DailyDisplayType > 1 and not (self.db.DisableDailysNoMaxLevel and _G.UnitLevel("player") < MAX_PLAYER_LEVEL) ) then
		local daily;
		
		if( self.db.DailyDisplayType == 2 ) then
			daily = _G.GetMaxDailyQuests() - _G.GetDailyQuestsCompleted();
		else
			daily = _G.GetDailyQuestsCompleted();
		end
		
		if( self.db.ShowNumMaxDailys ) then
			daily = daily.."/".._G.GetMaxDailyQuests();
		end
		
		if( self.db.EncolorDailysBy == 2 ) then
			daily = ("|cff1898F8%s|r"):format(daily);
		end
		
		self.Feed.text = self.Feed.text.."-"..daily;
	end
end

-----------------------
-- UpdateTooltip
-----------------------



function iQuest:UpdateTooltip()
	Tooltip:Clear();
	Tooltip:SetColumnLayout(3, "LEFT", "LEFT", "LEFT");
	
	local line;
	
	if( self.zaACM ) then
		local _, Name, _, Completed, _, _, _, _, _, Image, _ = _G.GetAchievementInfo(self.zaACM);
		local _, _, _, current, needed, _, _, _, _, _ = _G.GetAchievementCriteriaInfo(self.zaACM, 1);
		
		if( Name and Image and current and needed ) then
			line = Tooltip:AddHeader(
				"|T"..Image..":26:26|t",
			Name,
				("%d/%d"):format(current, needed)
			);
			
		end
		
	end
	
	line = Tooltip:AddLine("");
	Tooltip:SetCell(line, 1, "|cffffff00The tooltip is currently very BETA!|r", nil, "CENTER", 3);
end